'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const common = require('@graphql-toolkit/common');
const graphql = require('graphql');
const relayCompiler = require('relay-compiler');
const SkipRedundantNodesTransform = require('relay-compiler/lib/transforms/SkipRedundantNodesTransform');
const InlineFragmentsTransform = require('relay-compiler/lib/transforms/InlineFragmentsTransform');
const ApplyFragmentArgumentTransform = require('relay-compiler/lib/transforms/ApplyFragmentArgumentTransform');
const FlattenTransform = require('relay-compiler/lib/transforms/FlattenTransform');

const RelayCreate = require('relay-compiler/lib/core/Schema');
const GraphQLCompilerContext = require('relay-compiler/lib/core/GraphQLCompilerContext');
const { print } = require('relay-compiler/lib/core/GraphQLIRPrinter');
const plugin = (schema, documents, 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
config) => {
    // @TODO way for users to define directives they use, otherwise relay will throw an unknown directive error
    // Maybe we can scan the queries and add them dynamically without users having to do some extra stuff
    // transformASTSchema creates a new schema instance instead of mutating the old one
    const adjustedSchema = RelayCreate.create(common.printSchemaWithDirectives(relayCompiler.transformASTSchema(schema, [
        /* GraphQL */ `
          directive @connection(key: String!) on FIELD
          directive @client on FIELD
        `,
    ])));
    const documentAsts = documents.reduce((prev, v) => {
        return [...prev, ...v.content.definitions];
    }, []);
    const relayDocuments = relayCompiler.Parser.transform(adjustedSchema, documentAsts);
    const fragmentCompilerContext = new GraphQLCompilerContext(adjustedSchema).addAll(relayDocuments);
    const fragmentDocuments = fragmentCompilerContext
        .applyTransforms([ApplyFragmentArgumentTransform.transform, FlattenTransform.transformWithOptions({ flattenAbstractTypes: false }), SkipRedundantNodesTransform.transform])
        .documents()
        .filter(doc => doc.kind === 'Fragment');
    const queryCompilerContext = new GraphQLCompilerContext(adjustedSchema)
        .addAll(relayDocuments)
        .applyTransforms([ApplyFragmentArgumentTransform.transform, InlineFragmentsTransform.transform, FlattenTransform.transformWithOptions({ flattenAbstractTypes: false }), SkipRedundantNodesTransform.transform]);
    const newQueryDocuments = queryCompilerContext.documents().map(doc => ({
        filePath: 'optimized by relay',
        content: graphql.parse(print(adjustedSchema, doc)),
    }));
    const newDocuments = [
        ...fragmentDocuments.map(doc => ({
            filePath: 'optimized by relay',
            content: graphql.parse(print(adjustedSchema, doc)),
        })),
        ...newQueryDocuments,
    ];
    documents.splice(0, documents.length);
    documents.push(...newDocuments);
    return {
        content: '',
    };
};

exports.plugin = plugin;
